@@ -1,17 +1,43 @@
-{-# LANGUAGE DeriveDataTypeable, PatternGuards, TemplateHaskell #-}
+-- TODO: knock out these warnings
+{-# OPTIONS_GHC -fno-warn-name-shadowing #-}
+{-# OPTIONS_GHC -fno-warn-unused-matches #-}
+{-# OPTIONS_GHC -fno-warn-type-defaults #-}
+{-# OPTIONS_GHC -fno-warn-missing-signatures #-}
 
-module SKI (SKI(..),ski,parse) where
+{-# LANGUAGE CPP                #-}
+{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE PatternGuards      #-}
+{-# LANGUAGE TemplateHaskell    #-}
+
+module SKI
+  ( SKI(..)
+  , ski
+  , parse
+  , bracksP
+  , obrackP
+  , cbrackP
+  ) where
+
+import qualified Control.Monad.Fail           as Fail
+import           Data.Generics                (Data)
+import           Data.Typeable                (Typeable)
+import           Language.Haskell.Meta        (parseExp, parsePat)
+import           Language.Haskell.Meta.Utils  (cleanNames, ppDoc, unsafeRunQ)
+import           Language.Haskell.TH.Lib      hiding (parensP)
+import           Language.Haskell.TH.Ppr
+import           Language.Haskell.TH.Quote
+import           Language.Haskell.TH.Syntax
+import           Text.ParserCombinators.ReadP
+import           Text.PrettyPrint             (render)
+
+-- TODO: narrow type & move to shared module
+quoteTypeNotImplemented :: Fail.MonadFail m => String -> m a
+quoteTypeNotImplemented = fail . ("type quoter not implemented: " ++)
+
+-- TODO: narrow type & move to shared module
+quoteDecNotImplemented :: Fail.MonadFail m => String -> m a
+quoteDecNotImplemented = fail . ("dec quoter not implemented: " ++ )
 
-import Language.Haskell.Meta (parseExp, parsePat)
-import Language.Haskell.TH.Lib hiding (parensP)
-import Language.Haskell.TH.Ppr
-import Language.Haskell.TH.Quote
-import Language.Haskell.TH.Syntax
-import Language.Haskell.Meta.Utils (cleanNames, ppDoc, unsafeRunQ)
-import Text.ParserCombinators.ReadP
-import Data.Typeable(Typeable)
-import Data.Generics(Data)
-import Text.PrettyPrint(render)
 
 data SKI = S | K | I | E Exp | SKI :$ SKI
   deriving (Eq,Data,Typeable)
@@ -37,18 +63,24 @@ eval0  x                     = x
 
 ski :: QuasiQuoter
 ski = QuasiQuoter
-        {quoteExp = skiExpQ
-        ,quotePat = skiPatQ}
+  { quoteExp = skiExpQ
+  , quotePat = skiPatQ
+  , quoteType = quoteTypeNotImplemented
+  , quoteDec = quoteDecNotImplemented
+  }
 
 instance Lift SKI where
   lift = liftSKI
+#if MIN_VERSION_template_haskell(2,16,0)
+  liftTyped = unsafeTExpCoerce . lift -- TODO: the right way?
+#endif
 
 liftSKI (E e) = return e
 liftSKI a     = go a
-  where go S = [|S|]
-        go K = [|K|]
-        go I = [|I|]
-        go (E e) = [|E e|]
+  where go S      = [|S|]
+        go K      = [|K|]
+        go I      = [|I|]
+        go (E e)  = [|E e|]
         go (x:$y) = [|$(go x) :$ $(go y)|]
 
 instance Show SKI where
@@ -64,7 +96,7 @@ instance Show SKI where
 
 skiExpQ :: String -> ExpQ
 skiExpQ s = case run s of
-              [] -> fail "ski: parse error"
+              []  -> fail "ski: parse error"
               e:_ -> lift (cleanNames e)
 
 skiPatQ :: String -> PatQ
@@ -74,7 +106,7 @@ skiPatQ s = do
             . pprint
               . cleanNames) e
   case p of
-    Left e -> fail e
+    Left e  -> fail e
     Right p -> return p
 
 -- ghci> parse "S(SS)IK(SK)"
