@@ -1,25 +1,29 @@
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE TemplateHaskell, RankNTypes, StandaloneDeriving,
-  DeriveDataTypeable, PatternGuards, FlexibleContexts, FlexibleInstances,
-  TypeSynonymInstances #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-} -- TODO
+{-# LANGUAGE CPP                  #-}
+{-# LANGUAGE DeriveDataTypeable   #-}
+{-# LANGUAGE FlexibleContexts     #-}
+{-# LANGUAGE FlexibleInstances    #-}
+{-# LANGUAGE PatternGuards        #-}
+{-# LANGUAGE RankNTypes           #-}
+{-# LANGUAGE StandaloneDeriving   #-}
+{-# LANGUAGE TemplateHaskell      #-}
+{-# LANGUAGE TypeSynonymInstances #-}
 
 -- | This module is a staging ground
 -- for to-be-organized-and-merged-nicely code.
 
 module Language.Haskell.Meta.Utils where
 
-import Data.List (findIndex)
-import Data.Typeable
-import Data.Generics hiding(Fixity)
-import Language.Haskell.Meta
-import System.IO.Unsafe(unsafePerformIO)
-import Language.Haskell.Exts.Pretty(prettyPrint)
-import Language.Haskell.TH.Quote
-import Language.Haskell.TH.Syntax
-import Language.Haskell.TH.Lib
-import Language.Haskell.TH.Ppr
-import Text.PrettyPrint
 import Control.Monad
+import Data.Generics                hiding (Fixity)
+import Data.List                    (findIndex)
+import Language.Haskell.Exts.Pretty (prettyPrint)
+import Language.Haskell.Meta
+import Language.Haskell.TH.Lib      hiding (cxt)
+import Language.Haskell.TH.Ppr
+import Language.Haskell.TH.Syntax
+import System.IO.Unsafe             (unsafePerformIO)
+import Text.PrettyPrint
 
 -----------------------------------------------------------------------------
 
@@ -32,7 +36,7 @@ cleanNames = everywhere (mkT cleanName)
           | otherwise = (mkName . nameBase) n
         isNameU :: Name -> Bool
         isNameU (Name _ (NameU _)) = True
-        isNameU _ = False
+        isNameU _                  = False
 
 
 -- | The type passed in must have a @Show@ instance which
@@ -41,7 +45,7 @@ cleanNames = everywhere (mkT cleanName)
 --  but useful in general.
 pretty :: (Show a) => a -> String
 pretty a = case parseHsExp (show a) of
-            Left _ -> []
+            Left _  -> []
             Right e -> prettyPrint e
 
 
@@ -63,15 +67,6 @@ instance Show TypeQ where show = show . cleanNames . unsafeRunQ
 instance Show (Q String) where show = unsafeRunQ
 instance Show (Q Doc) where show = show . unsafeRunQ
 
-#if !MIN_VERSION_th_orphans(0,12,0)
-#if MIN_VERSION_base(4,7,0)
-deriving instance Typeable Q
-#else
-deriving instance Typeable1 Q
-#endif
-deriving instance Typeable QuasiQuoter
-#endif
-
 -- | @unsafeRunQ = unsafePerformIO . runQ@
 unsafeRunQ :: Q a -> a
 unsafeRunQ = unsafePerformIO . runQ
@@ -81,16 +76,16 @@ nameToRawCodeStr :: Name -> String
 nameToRawCodeStr n =
   let s = showNameParens n
   in case nameSpaceOf n of
-      Just VarName -> "'"++s
-      Just DataName -> "'"++s
+      Just VarName   -> "'"++s
+      Just DataName  -> "'"++s
       Just TcClsName -> "''"++s
-      _ -> concat ["(mkName \"", filter (/='"') s, "\")"]
+      _              -> concat ["(mkName \"", filter (/='"') s, "\")"]
   where showNameParens :: Name -> String
-        showNameParens n =
-          let nb = nameBase n
+        showNameParens n' =
+          let nb = nameBase n'
           in case nb of
             (c:_) | isSym c -> concat ["(",nb,")"]
-            _  -> nb
+            _               -> nb
         isSym :: Char -> Bool
         isSym = (`elem` ("><.\\/!@#$%^&*-+?:|" :: [Char]))
 
@@ -114,7 +109,7 @@ a |->| b = appT (appT arrowT a) b
 
 unForall :: Type -> Type
 unForall (ForallT _ _ t) = t
-unForall t = t
+unForall t               = t
 
 functionT :: [TypeQ] -> TypeQ
 functionT = foldl1 (|->|)
@@ -130,6 +125,8 @@ myNames = let xs = fmap (:[]) ['a'..'z']
            in fmap mkName (concat ys)
 
 -- | Generalisation of renameTs
+renameThings :: (t1 -> t2 -> a1 -> (a2, t1, t2))
+             -> t1 -> t2 -> [a2] -> [a1] -> ([a2], t1, t2)
 renameThings _ env new acc [] = (reverse acc, env, new)
 renameThings f env new acc (t:ts) =
   let (t', env', new') = f env new t
@@ -146,7 +143,7 @@ renameTs = renameThings renameT
 -- the fresh names list, and add this translation to the returned list.
 -- The fresh names list should be infinite; myNames is a good example.
 renameT :: [(Name, Name)] -> [Name] -> Type -> (Type, [(Name,Name)], [Name])
-renameT env [] _ = error "renameT: ran out of names!"
+renameT _env [] _ = error "renameT: ran out of names!"
 renameT env (x:new) (VarT n)
  | Just n' <- lookup n env = (VarT n',env,x:new)
  | otherwise = (VarT x, (n,x):env, new)
@@ -165,20 +162,10 @@ renameT env new (ForallT ns cxt t) =
     in (ForallT ns'' cxt' t', env4, new4)
   where
     unVarT (VarT n) = PlainTV n
+    unVarT ty       = error $ "renameT: unVarT: TODO for" ++ show ty
     renamePreds = renameThings renamePred
-
-#if MIN_VERSION_template_haskell(2,10,0)
     renamePred = renameT
-#else
-    renamePred env new (ClassP n ts) = let
-        (ts', env', new') = renameTs env new [] ts
-      in (ClassP (normaliseName n) ts', env', new')
-
-    renamePred env new (EqualP t1 t2) = let
-        (t1', env1, new1) = renameT env new t1
-        (t2', env2, new2) = renameT env1 new1 t2
-      in (EqualP t1' t2', env2, new2)
-#endif
+renameT _ _ t = error $ "renameT: TODO for " ++ show t
 
 -- | Remove qualification, etc.
 normaliseName :: Name -> Name
@@ -221,6 +208,10 @@ conTypes (NormalC _ sts) = fmap strictTypeTy sts
 conTypes (RecC    _ vts) = fmap varStrictTypeTy vts
 conTypes (InfixC t _ t') = fmap strictTypeTy [t,t']
 conTypes (ForallC _ _ c) = conTypes c
+conTypes c               = error $ "conTypes: TODO for " ++ show c
+-- TODO
+            -- (GadtC _ _ _)
+            -- (RecGadtC _ _ _)
 
 
 conToConType :: Type -> Con -> Type
@@ -230,61 +221,61 @@ conToConType ofType con = foldr (\a b -> AppT (AppT ArrowT a) b) ofType (conType
 
 decCons :: Dec -> [Con]
 #if MIN_VERSION_template_haskell(2,11,0)
-decCons (DataD _ _ _ _ cons _) = cons
+decCons (DataD _ _ _ _ cons _)   = cons
 decCons (NewtypeD _ _ _ _ con _) = [con]
 #else
-decCons (DataD _ _ _ cons _) = cons
-decCons (NewtypeD _ _ _ con _) = [con]
+decCons (DataD _ _ _ cons _)     = cons
+decCons (NewtypeD _ _ _ con _)   = [con]
 #endif
-decCons _ = []
+decCons _                        = []
 
 
 decTyVars :: Dec -> [TyVarBndr]
 #if MIN_VERSION_template_haskell(2,11,0)
-decTyVars (DataD _ _ ns _ _ _) = ns
+decTyVars (DataD _ _ ns _ _ _)    = ns
 decTyVars (NewtypeD _ _ ns _ _ _) = ns
 #else
-decTyVars (DataD _ _ ns _ _) = ns
-decTyVars (NewtypeD _ _ ns _ _) = ns
+decTyVars (DataD _ _ ns _ _)      = ns
+decTyVars (NewtypeD _ _ ns _ _)   = ns
 #endif
-decTyVars (TySynD _ ns _) = ns
-decTyVars (ClassD _ _ ns _ _) = ns
-decTyVars _ = []
+decTyVars (TySynD _ ns _)         = ns
+decTyVars (ClassD _ _ ns _ _)     = ns
+decTyVars _                       = []
 
 
 decName :: Dec -> Maybe Name
-decName (FunD n _) = Just n
+decName (FunD n _)             = Just n
 #if MIN_VERSION_template_haskell(2,11,0)
-decName (DataD _ n _ _ _ _) = Just n
+decName (DataD _ n _ _ _ _)    = Just n
 decName (NewtypeD _ n _ _ _ _) = Just n
 #else
-decName (DataD _ n _ _ _) = Just n
-decName (NewtypeD _ n _ _ _) = Just n
+decName (DataD _ n _ _ _)      = Just n
+decName (NewtypeD _ n _ _ _)   = Just n
 #endif
-decName (TySynD n _ _) = Just n
-decName (ClassD _ n _ _ _) = Just n
-decName (SigD n _) = Just n
-decName (ForeignD fgn) = Just (foreignName fgn)
-decName _ = Nothing
+decName (TySynD n _ _)         = Just n
+decName (ClassD _ n _ _ _)     = Just n
+decName (SigD n _)             = Just n
+decName (ForeignD fgn)         = Just (foreignName fgn)
+decName _                      = Nothing
 
 
 foreignName :: Foreign -> Name
 foreignName (ImportF _ _ _ n _) = n
-foreignName (ExportF _ _ n _) = n
+foreignName (ExportF _ _ n _)   = n
 
 
 unwindT :: Type -> [Type]
 unwindT = go
   where go :: Type -> [Type]
-        go (ForallT _ _ t) = go t
+        go (ForallT _ _ t)           = go t
         go (AppT (AppT ArrowT t) t') = t : go t'
-        go _ = []
+        go _                         = []
 
 
 unwindE :: Exp -> [Exp]
 unwindE = go []
   where go acc (e `AppE` e') = go (e':acc) e
-        go acc e = e:acc
+        go acc e             = e:acc
 
 
 -- | The arity of a Type.
@@ -308,42 +299,57 @@ typeToName t
 -- | Randomly useful.
 nameSpaceOf :: Name -> Maybe NameSpace
 nameSpaceOf (Name _ (NameG ns _ _)) = Just ns
-nameSpaceOf _ = Nothing
+nameSpaceOf _                       = Nothing
 
 conName :: Con -> Name
-conName (RecC n _) = n
-conName (NormalC n _) = n
-conName (InfixC _ n _) = n
+conName (RecC n _)        = n
+conName (NormalC n _)     = n
+conName (InfixC _ n _)    = n
 conName (ForallC _ _ con) = conName con
+conName c                 = error $ "conName: TODO for" ++ show c
+-- TODO
+            -- (GadtC _ _ _)
+            -- (RecGadtC _ _ _)
 
 recCName :: Con -> Maybe Name
 recCName (RecC n _) = Just n
-recCName _ = Nothing
+recCName _          = Nothing
 
 dataDCons :: Dec -> [Con]
 #if MIN_VERSION_template_haskell(2,11,0)
 dataDCons (DataD _ _ _ _ cons _) = cons
 #else
-dataDCons (DataD _ _ _ cons _) = cons
+dataDCons (DataD _ _ _ cons _)   = cons
 #endif
-dataDCons _ = []
+dataDCons _                      = []
 
 fromDataConI :: Info -> Q (Maybe Exp)
 #if MIN_VERSION_template_haskell(2,11,0)
-fromDataConI (DataConI dConN ty tyConN) =
+fromDataConI (DataConI dConN ty _tyConN) =
+  let n = arityT ty
+  in replicateM n (newName "a")
+      >>= \ns -> return (Just (LamE
+                    [ConP dConN (fmap VarP ns)]
+#if MIN_VERSION_template_haskell(2,16,0)
+                    (TupE $ fmap (Just . VarE) ns)
 #else
-fromDataConI (DataConI dConN ty tyConN fxty) =
+                    (TupE $ fmap VarE ns)
 #endif
+                    ))
+#else
+fromDataConI (DataConI dConN ty _tyConN _fxty) =
   let n = arityT ty
   in replicateM n (newName "a")
       >>= \ns -> return (Just (LamE
                     [ConP dConN (fmap VarP ns)]
                     (TupE $ fmap VarE ns)))
+
+#endif
 fromDataConI _ = return Nothing
 
 fromTyConI :: Info -> Maybe Dec
 fromTyConI (TyConI dec) = Just dec
-fromTyConI _ = Nothing
+fromTyConI _            = Nothing
 
 mkFunD :: Name -> [Pat] -> Exp -> Dec
 mkFunD f xs e = FunD f [Clause xs (NormalB e) []]
