@@ -1,13 +1,12 @@
-{-# LANGUAGE TemplateHaskell, QuasiQuotes #-}
-
-import Control.Monad (when)
+{-# LANGUAGE QuasiQuotes     #-}
+{-# LANGUAGE TemplateHaskell #-}
 
 import qualified BF
 import qualified Hs
 import qualified HsHere
 import qualified SKI
 
-import SKI (SKI(S, K, I, (:$)))
+import SKI (SKI ((:$), I, K, S))
 
 -- Very dumb test framework
 shouldBe :: (Show a, Eq a) => a -> a -> IO ()
@@ -27,7 +26,7 @@ a x = [HsHere.here| random "text" $(x + 1)
 
 hereTest :: IO ()
 hereTest = do
- a 3 `shouldBe` (" random \"text\" "++ show (3 + 1) ++"\n  something else")
+ a 3 `shouldBe` (" random \"text\" "++ show (3 + 1 :: Int) ++"\n  something else")
 
 -- TODO: better test exercising the bf quasiquoter
 
@@ -37,7 +36,7 @@ bfTest = do
 
 hsTest :: IO ()
 hsTest = do
-  (\ [Hs.hs|a@(x,_)|] -> [Hs.hs|(a,x)|]) (42,88) `shouldBe` ((42,88),42)
+  (\ [Hs.hs|b@(x,_)|] -> [Hs.hs|(b,x)|]) (42 :: Int,88 :: Int) `shouldBe` ((42,88),42)
 
 -- TODO: better test exercising the ski quasiquoter
 
